public class Isograph {
  // Simulation des Graphenisomorphie-Problems

  static int n = 7; // Anzahl der Knoten
  static int[] sigma  = {4, 6, 3, 2, 5, 0, 1};

  static void GraphAufbau (Graph g) {
    g.verbinde(0, 1); g.verbinde(0, 6);
    g.verbinde(1, 5); g.verbinde(2, 3);
    g.verbinde(3, 5); g.verbinde(4, 3);
    g.verbinde(5, 6);
    } // Ende GraphAufbau

  static Graph permutiert (Graph g, int[] p) {
    // erzeugt zu g isomorphen Graph
    int n = g.AnzahlKnoten();
    Graph neu = new Graph(n, false);
    for (int i = 0; i < n - 1; i++)
      for (int j = i + 1; j < n; j++)
        if (g.istBenachbart(i, j))
          neu.verbinde(p[i], p[j]);
    return neu;
    } // Ende permutiert

  static int[] umgekehrt (int[] p) {
    // kehrt Permutation p um
    int n = p.length;
    int[] neu = new int[n];
    for (int i = 0; i < n; i++)
      for (int j = 0; j < n; j++)
        if (i == p[j]) neu[i] = j;
    return neu;
    } // Ende umgekehrt

  static int[] verkettet (int[] p, int[] q) {
    // verkettet Permutationen p mit q
    int n = q.length;
    int[] neu = new int[n];
    for (int i = 0; i < n; i++) neu[i] = p[q[i]];
    return neu;
    } // Ende verkettet

  static void vertausche (int[] v, int i, int j)
    {int m = v[i]; v[i] = v[j]; v[j] = m;}

  static int[] Zufallsperm (int n) {
    // erzeugt Zufallspermutation
    int[] perm = new int[n];
    for (int i = 0; i < n; i++) perm[i] = i;
    for (int k = n - 1; k > 0; k--) {
      int zz = (int) (Math.random() * k);
      vertausche(perm, zz, k);
      } // Ende for
    return perm;
    } // Ende Zufallsperm

  static void Meldung (boolean gleich) {
    if (gleich)
      System.out.println(" Richtig!");
    else
      System.out.println(" Falsch");
    } // Ende Meldung

  static int[] tau (int bit, int[] perm) {
    // Antwort-Permutation des Proponenten A
    if (bit == 0)
      return umgekehrt(perm);
    else
      return verkettet(sigma, umgekehrt(perm));
    } // Ende tau

/*
  public static void main (String[] xy) {
    Graph G0 = new Graph(n, false);
    GraphAufbau(G0);
    Graph G1 = permutiert(G0, sigma);
    String text = "\n GRAPH-ISOMORPHIE (Version A)\n\n";
    System.out.println(text + G0.Ausgabe() + G1.Ausgabe());
    for (int runde = 0; runde < 10; runde++) {
      int[] p = Zufallsperm(n);
      Graph H = permutiert(G0, p);
      // System.out.println(H.Ausgabe());
      int k = (int) (Math.random() * 2);
      Graph G = (k == 0) ? G0 : G1;
      Meldung(G.istGleich(permutiert(H, tau(k, p))));
      } // Ende for
    } // Ende main
*/
  public static void main (String[] xy) {
    Graph G0 = new Graph(n, false);
    GraphAufbau(G0);
    Graph G1 = permutiert(G0, sigma);
    String text = "\n GRAPH-ISOMORPHIE (Version X)\n\n";
    System.out.println(text + G0.Ausgabe() + G1.Ausgabe());
    int b = (int) (Math.random() * 2);
    for (int runde = 0; runde < 10; runde++) {
      Graph G = (b == 0) ? G0 : G1;
      int[] p = Zufallsperm(n);
      Graph H = permutiert(G, p);
      // System.out.println(H.Ausgabe());
      int k = (int) (Math.random() * 2);
      if (k == b)
        Meldung(G.istGleich(permutiert(H, umgekehrt(p))));
      else
        System.out.println(" " + k + " Falsch");
      } // Ende for
    } // Ende main

  } // Ende Isograph










